@@ -1009,7 +1009,6 @@ static int fix_win_file_permissions(const char *file)
 	ACL* pOldDACL;
 	SECURITY_DESCRIPTOR* pSD = NULL;
 	EXPLICIT_ACCESS ea = { 0 };
-	BOOL isWellKnownSID = FALSE;
 	PSID pSid = NULL;
 
 	GetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
@@ -1029,7 +1028,7 @@ static int fix_win_file_permissions(const char *file)
 	ea.grfInheritance = CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE;
 	ea.Trustee.TrusteeType = TRUSTEE_IS_UNKNOWN;
 	ACL* pNewDACL = 0;
-	DWORD err = SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
+	SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
 	if (pNewDACL)
 	{
 		SetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
@@ -1445,7 +1444,7 @@ out:
 
 void backup_fix_ddl(void);
 
-lsn_t get_current_lsn(MYSQL *connection)
+static lsn_t get_current_lsn(MYSQL *connection)
 {
 	static const char lsn_prefix[] = "\nLog sequence number ";
 	lsn_t lsn = 0;
@@ -1783,7 +1782,7 @@ apply_log_finish()
 bool
 copy_back()
 {
-	bool ret = false;
+	bool ret;
 	datadir_iter_t *it = NULL;
 	datadir_node_t node;
 	char *dst_dir;
