@@ -1,4 +1,5 @@
 /* Copyright (C) 2013, 2015, Alexey Botchkov and SkySQL Ab
+   Copyright (c) 2019, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
 
 #define PLUGIN_VERSION 0x104
-#define PLUGIN_STR_VERSION "1.4.13"
+#define PLUGIN_STR_VERSION "1.4.8"
 
 #define _my_thread_var loc_thread_var
 
@@ -139,7 +140,6 @@ static int loc_file_errno;
 #define logger_write loc_logger_write
 #define logger_rotate loc_logger_rotate
 #define logger_init_mutexts loc_logger_init_mutexts
-#define logger_time_to_rotate loc_logger_time_to_rotate
 
 
 static size_t loc_write(File Filedes, const uchar *Buffer, size_t Count)
@@ -327,10 +327,6 @@ struct connection_info
   char query_buffer[1024];
   time_t query_time;
   int log_always;
-  char proxy[64];
-  int proxy_length;
-  char proxy_host[64];
-  int proxy_host_length;
 };
 
 #define DEFAULT_FILENAME_LEN 16
@@ -554,22 +550,22 @@ static struct st_mysql_show_var audit_status[]=
   {0,0,0}
 };
 
-#ifdef HAVE_PSI_INTERFACE
+#if defined(HAVE_PSI_INTERFACE) && !defined(FLOGGER_NO_PSI)
+/* These belong to the service initialization */
 static PSI_mutex_key key_LOCK_operations;
+static PSI_mutex_key key_LOCK_atomic;
+static PSI_mutex_key key_LOCK_bigbuffer;
 static PSI_mutex_info mutex_key_list[]=
 {
   { &key_LOCK_operations, "SERVER_AUDIT_plugin::lock_operations",
-    PSI_FLAG_GLOBAL}
-#ifndef FLOGGER_NO_PSI
-  ,
+    PSI_FLAG_GLOBAL},
   { &key_LOCK_atomic, "SERVER_AUDIT_plugin::lock_atomic",
     PSI_FLAG_GLOBAL},
   { &key_LOCK_bigbuffer, "SERVER_AUDIT_plugin::lock_bigbuffer",
     PSI_FLAG_GLOBAL}
-#endif /*FLOGGER_NO_PSI*/
 };
-#endif /*HAVE_PSI_INTERFACE*/
-static mysql_prlock_t lock_operations;
+#endif
+static mysql_mutex_t lock_operations;
 static mysql_mutex_t lock_atomic;
 static mysql_mutex_t lock_bigbuffer;
 
@@ -709,8 +705,6 @@ static char *coll_search(struct user_coll *c, const char *n, size_t len)
 {
   struct user_name un;
   struct user_name *found;
-  if (!c->n_users)
-    return 0;
   un.name_len= len;
   un.name= (char *) n;
   found= (struct user_name*)  bsearch(&un, c->users, c->n_users,
@@ -741,8 +735,7 @@ static int coll_insert(struct user_coll *c, char *n, size_t len)
 
 static void coll_sort(struct user_coll *c)
 {
-  if (c->n_users)
-    qsort(c->users, c->n_users, sizeof(c->users[0]), cmp_users);
+  qsort(c->users, c->n_users, sizeof(c->users[0]), cmp_users);
 }
 
 
@@ -773,7 +766,7 @@ static int user_coll_fill(struct user_coll *c, char *users,
         ADD_ATOMIC(internal_stop_logging, 1);
         CLIENT_ERROR(1, "User '%.*b' was removed from the"
             " server_audit_excl_users.",
-            MYF(ME_JUST_WARNING), (int) cmp_length, users);
+            MYF(ME_WARNING), (int) cmp_length, users);
         ADD_ATOMIC(internal_stop_logging, -1);
         blank_user(cmp_user);
         refill_cmp_coll= 1;
@@ -782,7 +775,7 @@ static int user_coll_fill(struct user_coll *c, char *users,
       {
         ADD_ATOMIC(internal_stop_logging, 1);
         CLIENT_ERROR(1, "User '%.*b' is in the server_audit_incl_users, "
-            "so wasn't added.", MYF(ME_JUST_WARNING), (int) cmp_length, users);
+            "so wasn't added.", MYF(ME_WARNING), (int) cmp_length, users);
         ADD_ATOMIC(internal_stop_logging, -1);
         remove_user(users);
         continue;
@@ -819,7 +812,6 @@ enum sa_keywords
   SQLCOM_DML,
   SQLCOM_GRANT,
   SQLCOM_CREATE_USER,
-  SQLCOM_ALTER_USER,
   SQLCOM_CHANGE_MASTER,
   SQLCOM_CREATE_SERVER,
   SQLCOM_SET_OPTION,
@@ -858,8 +850,12 @@ struct sa_keyword keywords_to_skip[]=
 
 struct sa_keyword not_ddl_keywords[]=
 {
+  {4, "DROP", &function_word, SQLCOM_QUERY_ADMIN},
+  {4, "DROP", &procedure_word, SQLCOM_QUERY_ADMIN},
   {4, "DROP", &user_word, SQLCOM_DCL},
   {6, "CREATE", &user_word, SQLCOM_DCL},
+  {6, "CREATE", &function_word, SQLCOM_QUERY_ADMIN},
+  {6, "CREATE", &procedure_word, SQLCOM_QUERY_ADMIN},
   {6, "RENAME", &user_word, SQLCOM_DCL},
   {0, NULL, 0, SQLCOM_DDL}
 };
@@ -923,7 +919,6 @@ struct sa_keyword passwd_keywords[]=
 {
   {3, "SET", &password_word, SQLCOM_SET_OPTION},
   {5, "ALTER", &server_word, SQLCOM_ALTER_SERVER},
-  {5, "ALTER", &user_word, SQLCOM_ALTER_USER},
   {5, "GRANT", 0, SQLCOM_GRANT},
   {6, "CREATE", &user_word, SQLCOM_CREATE_USER},
   {6, "CREATE", &server_word, SQLCOM_CREATE_SERVER},
@@ -971,8 +966,7 @@ static void get_str_n(char *dest, int *dest_len, size_t dest_size,
   if (src_len >= dest_size)
     src_len= dest_size - 1;
 
-  if (src_len)
-    memcpy(dest, src, src_len);
+  memcpy(dest, src, src_len);
   dest[src_len]= 0;
   *dest_len= (int)src_len;
 }
@@ -1089,7 +1083,7 @@ static int start_logging()
                   "Could not create file '%s'.", alt_fname);
       is_active= 0;
       CLIENT_ERROR(1, "SERVER AUDIT plugin can't create file '%s'.",
-          MYF(ME_JUST_WARNING), alt_fname);
+          MYF(ME_WARNING), alt_fname);
       return 1;
     }
     error_header();
@@ -1137,13 +1131,9 @@ static void setup_connection_simple(struct connection_info *ci)
   ci->ip_length= 0;
   ci->query_length= 0;
   ci->header= 0;
-  ci->proxy_length= 0;
 }
 
 
-#define MAX_HOSTNAME 61
-#define USERNAME_LENGTH 384
-
 static void setup_connection_connect(struct connection_info *cn,
     const struct mysql_event_connection *event)
 {
@@ -1160,29 +1150,6 @@ static void setup_connection_connect(struct connection_info *cn,
   get_str_n(cn->ip, &cn->ip_length, sizeof(cn->ip),
             event->ip, event->ip_length);
   cn->header= 0;
-  if (event->proxy_user && event->proxy_user[0])
-  {
-    const char *priv_host= event->proxy_user +
-            sizeof(char[MAX_HOSTNAME+USERNAME_LENGTH+5]);
-    size_t priv_host_length;
-
-    if (mysql_57_started)
-    {
-      priv_host+= sizeof(size_t);
-      priv_host_length= *(size_t *) (priv_host + MAX_HOSTNAME);
-    }
-    else
-      priv_host_length= strlen(priv_host);
-
-
-    get_str_n(cn->proxy, &cn->proxy_length, sizeof(cn->proxy),
-              event->priv_user, event->priv_user_length);
-    get_str_n(cn->proxy_host, &cn->proxy_host_length,
-              sizeof(cn->proxy_host),
-              priv_host, priv_host_length);
-  }
-  else
-    cn->proxy_length= 0;
 }
 
 
@@ -1318,41 +1285,19 @@ static void change_connection(struct connection_info *cn,
             event->ip, event->ip_length);
 }
 
-/*
-  Write to the log
-
-  @param take_lock  If set, take a read lock (or write lock on rotate).
-                    If not set, the caller has a already taken a write lock
-*/
-
 static int write_log(const char *message, size_t len, int take_lock)
 {
   int result= 0;
   if (take_lock)
-  {
-    /* Start by taking a read lock */
-    mysql_prlock_rdlock(&lock_operations);
-  }
+    flogger_mutex_lock(&lock_operations);
 
   if (output_type == OUTPUT_FILE)
   {
-    if (logfile)
-    {
-      my_bool allow_rotate= !take_lock; /* Allow rotate if caller write lock */
-      if (take_lock && logger_time_to_rotate(logfile))
-      {
-        /* We have to rotate the log, change above read lock to write lock */
-        mysql_prlock_unlock(&lock_operations);
-        mysql_prlock_wrlock(&lock_operations);
-        allow_rotate= 1;
-      }
-      if (!(is_active= (logger_write_r(logfile, allow_rotate, message, len) ==
-                        (int) len)))
-      {
-        ++log_write_failures;
-        result= 1;
-      }
-    }
+    if (logfile &&
+        (is_active= (logger_write(logfile, message, len) == (int) len)))
+      goto exit;
+    ++log_write_failures;
+    result= 1;
   }
   else if (output_type == OUTPUT_SYSLOG)
   {
@@ -1360,8 +1305,9 @@ static int write_log(const char *message, size_t len, int take_lock)
            syslog_priority_codes[syslog_priority],
            "%s %.*s", syslog_info, (int) len, message);
   }
+exit:
   if (take_lock)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
   return result;
 }
 
@@ -1403,31 +1349,6 @@ static size_t log_header(char *message, size_t message_len,
 }
 
 
-static int log_proxy(const struct connection_info *cn,
-                     const struct mysql_event_connection *event)
-                   
-{
-  time_t ctime;
-  size_t csize;
-  char message[1024];
-
-  (void) time(&ctime);
-  csize= log_header(message, sizeof(message)-1, &ctime,
-                    servhost, servhost_len,
-                    cn->user, cn->user_length,
-                    cn->host, cn->host_length,
-                    cn->ip, cn->ip_length,
-                    event->thread_id, 0, "PROXY_CONNECT");
-  csize+= my_snprintf(message+csize, sizeof(message) - 1 - csize,
-    ",%.*s,`%.*s`@`%.*s`,%d", cn->db_length, cn->db,
-                     cn->proxy_length, cn->proxy,
-                     cn->proxy_host_length, cn->proxy_host,
-                     event->status);
-  message[csize]= '\n';
-  return write_log(message, csize + 1, 1);
-}
-
-
 static int log_connection(const struct connection_info *cn,
                           const struct mysql_event_connection *event,
                           const char *type)
@@ -1510,6 +1431,7 @@ static size_t escape_string_hide_passwords(const char *str, unsigned int len,
   const char *res_start= result;
   const char *res_end= result + result_len - 2;
   size_t d_len;
+  char b_char;
 
   while (len)
   {
@@ -1547,28 +1469,27 @@ static size_t escape_string_hide_passwords(const char *str, unsigned int len,
 
       if (*next_s)
       {
-        const char b_char= *next_s++;
-        memset(result + d_len, '*', 5);
+        memmove(result + d_len, "*****", 5);
         result+= d_len + 5;
-
-        while (*next_s)
-        {
-          if (*next_s == b_char)
-          {
-            ++next_s;
-            break;
-          }
-          if (*next_s == '\\')
-          {
-            if (next_s[1])
-              next_s++;
-          }
-          next_s++;
-        }
+        b_char= *(next_s++);
       }
       else
         result+= d_len;
 
+      while (*next_s)
+      {
+        if (*next_s == b_char)
+        {
+          ++next_s;
+          break;
+        }
+        if (*next_s == '\\')
+        {
+          if (next_s[1])
+            next_s++;
+        }
+        next_s++;
+      }
       len-= (uint)(next_s - str);
       str= next_s;
       continue;
@@ -1576,23 +1497,19 @@ static size_t escape_string_hide_passwords(const char *str, unsigned int len,
 no_password:
     if (result >= res_end)
       break;
-    else
+    if ((b_char= escaped_char(*str)))
     {
-      const char b_char= escaped_char(*str);
-      if (b_char)
-      {
-        if (result+1 >= res_end)
-          break;
-        *(result++)= '\\';
-        *(result++)= b_char;
-      }
-      else if (is_space(*str))
-        *(result++)= ' ';
-      else
-        *(result++)= *str;
-      str++;
-      len--;
+      if (result+1 >= res_end)
+        break;
+      *(result++)= '\\';
+      *(result++)= b_char;
     }
+    else if (is_space(*str))
+      *(result++)= ' ';
+    else
+      *(result++)= *str;
+    str++;
+    len--;
   }
   *result= 0;
   return result - res_start;
@@ -1600,32 +1517,27 @@ no_password:
 
 
 
-static int do_log_user(const char *name, int len,
-                       const char *proxy, int proxy_len, int take_lock)
+static int do_log_user(const char *name, int take_lock)
 {
+  size_t len;
   int result;
 
   if (!name)
     return 0;
+  len= strlen(name);
 
   if (take_lock)
-    mysql_prlock_rdlock(&lock_operations);
+    flogger_mutex_lock(&lock_operations);
 
   if (incl_user_coll.n_users)
-  {
-    result= coll_search(&incl_user_coll, name, len) != 0 ||
-            (proxy && coll_search(&incl_user_coll, proxy, proxy_len) != 0);
-  }
+    result= coll_search(&incl_user_coll, name, len) != 0;
   else if (excl_user_coll.n_users)
-  {
-    result= coll_search(&excl_user_coll, name, len) == 0 &&
-            (proxy && coll_search(&excl_user_coll, proxy, proxy_len) == 0);
-  }
+    result=  coll_search(&excl_user_coll, name, len) == 0;
   else
     result= 1;
 
   if (take_lock)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
   return result;
 }
 
@@ -1843,7 +1755,6 @@ do_log_query:
   {
     case SQLCOM_GRANT:
     case SQLCOM_CREATE_USER:
-    case SQLCOM_ALTER_USER:
       csize+= escape_string_hide_passwords(query, query_len,
                                            uh_buffer, uh_buffer_size,
                                            "IDENTIFIED", 10, "BY", 2, 0);
@@ -2162,9 +2073,7 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
   }
 
   if (event_class == MYSQL_AUDIT_GENERAL_CLASS && FILTER(EVENT_QUERY) &&
-      cn && (cn->log_always || do_log_user(cn->user, cn->user_length,
-                                           cn->proxy, cn->proxy_length,
-                                           1)))
+      cn && (cn->log_always || do_log_user(cn->user, 1)))
   {
     const struct mysql_event_general *event =
       (const struct mysql_event_general *) ev;
@@ -2184,8 +2093,7 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
   {
     const struct mysql_event_table *event =
       (const struct mysql_event_table *) ev;
-    if (do_log_user(event->user, (int) SAFE_STRLEN(event->user),
-                    cn->proxy, cn->proxy_length, 1))
+    if (do_log_user(event->user, 1))
     {
       switch (event->event_subclass)
       {
@@ -2218,8 +2126,6 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
     {
       case MYSQL_AUDIT_CONNECTION_CONNECT:
         log_connection(cn, event, event->status ? "FAILED_CONNECT": "CONNECT");
-        if (event->status == 0 && event->proxy_user && event->proxy_user[0])
-          log_proxy(cn, event);
         break;
       case MYSQL_AUDIT_CONNECTION_DISCONNECT:
         if (use_event_data_for_disconnect)
@@ -2229,8 +2135,6 @@ void auditing(MYSQL_THD thd, unsigned int event_class, const void *ev)
         break;
       case MYSQL_AUDIT_CONNECTION_CHANGE_USER:
         log_connection(cn, event, "CHANGEUSER");
-        if (event->proxy_user && event->proxy_user[0])
-          log_proxy(cn, event);
         break;
       default:;
     }
@@ -2517,11 +2421,11 @@ static int server_audit_init(void *p __attribute__((unused)))
   servhost_len= (uint)strlen(servhost);
 
   logger_init_mutexes();
-#ifdef HAVE_PSI_INTERFACE
+#if defined(HAVE_PSI_INTERFACE) && !defined(FLOGGER_NO_PSI)
   if (PSI_server)
     PSI_server->register_mutex("server_audit", mutex_key_list, 1);
 #endif
-  mysql_prlock_init(key_LOCK_operations, &lock_operations);
+  flogger_mutex_init(key_LOCK_operations, &lock_operations, MY_MUTEX_INIT_FAST);
   flogger_mutex_init(key_LOCK_operations, &lock_atomic, MY_MUTEX_INIT_FAST);
   flogger_mutex_init(key_LOCK_operations, &lock_bigbuffer, MY_MUTEX_INIT_FAST);
 
@@ -2609,7 +2513,7 @@ static int server_audit_deinit(void *p __attribute__((unused)))
     closelog();
 
   (void) free(big_buffer);
-  mysql_prlock_destroy(&lock_operations);
+  flogger_mutex_destroy(&lock_operations);
   flogger_mutex_destroy(&lock_atomic);
   flogger_mutex_destroy(&lock_bigbuffer);
 
@@ -2720,7 +2624,7 @@ static void update_file_path(MYSQL_THD thd,
   fprintf(stderr, "Log file name was changed to '%s'.\n", new_name);
 
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_wrlock(&lock_operations);
+    flogger_mutex_lock(&lock_operations);
 
   if (logging)
     log_current_query(thd);
@@ -2741,7 +2645,7 @@ static void update_file_path(MYSQL_THD thd,
       {
         error_header();
         fprintf(stderr, "Logging was disabled..\n");
-        CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_JUST_WARNING));
+        CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_WARNING));
       }
       goto exit_func;
     }
@@ -2752,7 +2656,7 @@ static void update_file_path(MYSQL_THD thd,
   file_path= path_buffer;
 exit_func:
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2768,9 +2672,9 @@ static void update_file_rotations(MYSQL_THD thd  __attribute__((unused)),
   if (!logging || output_type != OUTPUT_FILE)
     return;
 
-  mysql_prlock_wrlock(&lock_operations);
+  flogger_mutex_lock(&lock_operations);
   logfile->rotations= rotations;
-  mysql_prlock_unlock(&lock_operations);
+  flogger_mutex_unlock(&lock_operations);
 }
 
 
@@ -2786,9 +2690,9 @@ static void update_file_rotate_size(MYSQL_THD thd  __attribute__((unused)),
   if (!logging || output_type != OUTPUT_FILE)
     return;
 
-  mysql_prlock_wrlock(&lock_operations);
+  flogger_mutex_lock(&lock_operations);
   logfile->size_limit= file_rotate_size;
-  mysql_prlock_unlock(&lock_operations);
+  flogger_mutex_unlock(&lock_operations);
 }
 
 
@@ -2833,7 +2737,7 @@ static void update_incl_users(MYSQL_THD thd,
   char *new_users= (*(char **) save) ? *(char **) save : empty_str;
   size_t new_len= strlen(new_users) + 1;
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_wrlock(&lock_operations);
+    flogger_mutex_lock(&lock_operations);
   mark_always_logged(thd);
 
   if (new_len > sizeof(incl_user_buffer))
@@ -2847,7 +2751,7 @@ static void update_incl_users(MYSQL_THD thd,
   error_header();
   fprintf(stderr, "server_audit_incl_users set to '%s'.\n", incl_users);
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
 }
 
 
@@ -2858,7 +2762,7 @@ static void update_excl_users(MYSQL_THD thd  __attribute__((unused)),
   char *new_users= (*(char **) save) ? *(char **) save : empty_str;
   size_t new_len= strlen(new_users) + 1;
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_wrlock(&lock_operations);
+    flogger_mutex_lock(&lock_operations);
   mark_always_logged(thd);
 
   if (new_len > sizeof(excl_user_buffer))
@@ -2872,7 +2776,7 @@ static void update_excl_users(MYSQL_THD thd  __attribute__((unused)),
   error_header();
   fprintf(stderr, "server_audit_excl_users set to '%s'.\n", excl_users);
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
 }
 
 
@@ -2885,7 +2789,7 @@ static void update_output_type(MYSQL_THD thd,
     return;
 
   ADD_ATOMIC(internal_stop_logging, 1);
-  mysql_prlock_wrlock(&lock_operations);
+  flogger_mutex_lock(&lock_operations);
   if (logging)
   {
     log_current_query(thd);
@@ -2899,7 +2803,7 @@ static void update_output_type(MYSQL_THD thd,
 
   if (logging)
     start_logging();
-  mysql_prlock_unlock(&lock_operations);
+  flogger_mutex_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2929,9 +2833,9 @@ static void update_syslog_priority(MYSQL_THD thd  __attribute__((unused)),
   if (syslog_priority == new_priority)
     return;
 
-  mysql_prlock_wrlock(&lock_operations);
+  flogger_mutex_lock(&lock_operations);
   mark_always_logged(thd);
-  mysql_prlock_unlock(&lock_operations);
+  flogger_mutex_unlock(&lock_operations);
   error_header();
   fprintf(stderr, "SysLog priority was changed from '%s' to '%s'.\n",
           syslog_priority_names[syslog_priority],
@@ -2950,13 +2854,13 @@ static void update_logging(MYSQL_THD thd,
 
   ADD_ATOMIC(internal_stop_logging, 1);
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_wrlock(&lock_operations);
+    flogger_mutex_lock(&lock_operations);
   if ((logging= new_logging))
   {
     start_logging();
     if (!logging)
     {
-      CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_JUST_WARNING));
+      CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_WARNING));
     }
     mark_always_logged(thd);
   }
@@ -2967,7 +2871,7 @@ static void update_logging(MYSQL_THD thd,
   }
 
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -2982,13 +2886,13 @@ static void update_mode(MYSQL_THD thd  __attribute__((unused)),
 
   ADD_ATOMIC(internal_stop_logging, 1);
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_wrlock(&lock_operations);
+    flogger_mutex_lock(&lock_operations);
   mark_always_logged(thd);
   error_header();
   fprintf(stderr, "Logging mode was changed from %d to %d.\n", mode, new_mode);
   mode= new_mode;
   if (!maria_55_started || !debug_server_started)
-    mysql_prlock_unlock(&lock_operations);
+    flogger_mutex_unlock(&lock_operations);
   ADD_ATOMIC(internal_stop_logging, -1);
 }
 
@@ -3003,14 +2907,14 @@ static void update_syslog_ident(MYSQL_THD thd  __attribute__((unused)),
   syslog_ident= syslog_ident_buffer;
   error_header();
   fprintf(stderr, "SYSYLOG ident was changed to '%s'\n", syslog_ident);
-  mysql_prlock_wrlock(&lock_operations);
+  flogger_mutex_lock(&lock_operations);
   mark_always_logged(thd);
   if (logging && output_type == OUTPUT_SYSLOG)
   {
     stop_logging();
     start_logging();
   }
-  mysql_prlock_unlock(&lock_operations);
+  flogger_mutex_unlock(&lock_operations);
 }
 
 
