@@ -437,6 +437,11 @@ buf_dump(
 
 	buf_dump_status(STATUS_INFO,
 			"Buffer pool(s) dump completed at %s", now);
+
+	/* Though dumping doesn't related to an incomplete load,
+	 we reset this to 0 here to indicate that a shutdown can also perform
+	 a dump */
+	export_vars.innodb_buffer_pool_load_incomplete = 0;
 }
 
 /*****************************************************************//**
@@ -535,7 +540,7 @@ buf_load()
 	buf_load_status(STATUS_INFO,
 			"Loading buffer pool(s) from %s", full_filename);
 
-	f = fopen(full_filename, "r");
+	f = fopen(full_filename, "r" STR_O_CLOEXEC);
 	if (f == NULL) {
 		buf_load_status(STATUS_INFO,
 				"Cannot open '%s' for reading: %s",
@@ -600,6 +605,8 @@ buf_load()
 
 	rewind(f);
 
+	export_vars.innodb_buffer_pool_load_incomplete = 1;
+
 	for (i = 0; i < dump_n && !SHUTTING_DOWN(); i++) {
 		fscanf_ret = fscanf(f, ULINTPF "," ULINTPF,
 				    &space_id, &page_no);
@@ -648,7 +655,7 @@ buf_load()
 		ut_sprintf_timestamp(now);
 		buf_load_status(STATUS_INFO,
 				"Buffer pool(s) load completed at %s"
-				" (%s was empty)", now, full_filename);
+				" (%s was empty or had errors)", now, full_filename);
 		return;
 	}
 
@@ -664,7 +671,7 @@ buf_load()
 	so all pages from a given tablespace are consecutive. */
 	ulint		cur_space_id = BUF_DUMP_SPACE(dump[0]);
 	fil_space_t*	space = fil_space_acquire_silent(cur_space_id);
-	page_size_t	page_size(space ? space->flags : 0);
+	ulint		zip_size = space ? space->zip_size() : 0;
 
 	/* JAN: TODO: MySQL 5.7 PSI
 #ifdef HAVE_PSI_STAGE_INTERFACE
@@ -688,16 +695,14 @@ buf_load()
 
 		if (this_space_id != cur_space_id) {
 			if (space != NULL) {
-				fil_space_release(space);
+				space->release();
 			}
 
 			cur_space_id = this_space_id;
 			space = fil_space_acquire_silent(cur_space_id);
 
 			if (space != NULL) {
-				const page_size_t	cur_page_size(
-					space->flags);
-				page_size.copy_from(cur_page_size);
+				zip_size = space->zip_size();
 			}
 		}
 
@@ -712,7 +717,7 @@ buf_load()
 
 		buf_read_page_background(
 			page_id_t(this_space_id, BUF_DUMP_PAGE(dump[i])),
-			page_size, true);
+			zip_size, true);
 
 		if (i % 64 == 63) {
 			os_aio_simulated_wake_handler_threads();
@@ -720,7 +725,7 @@ buf_load()
 
 		if (buf_load_abort_flag) {
 			if (space != NULL) {
-				fil_space_release(space);
+				space->release();
 			}
 			buf_load_abort_flag = FALSE;
 			ut_free(dump);
@@ -742,18 +747,39 @@ buf_load()
 
 		buf_load_throttle_if_needed(
 			&last_check_time, &last_activity_cnt, i);
+
+#ifdef UNIV_DEBUG
+		if ((i+1) >= srv_buf_pool_load_pages_abort) {
+			buf_load_abort_flag = 1;
+		}
+#endif
 	}
 
 	if (space != NULL) {
-		fil_space_release(space);
+		space->release();
 	}
 
 	ut_free(dump);
 
 	ut_sprintf_timestamp(now);
 
-	buf_load_status(STATUS_INFO,
+	if (i == dump_n) {
+		buf_load_status(STATUS_INFO,
 			"Buffer pool(s) load completed at %s", now);
+		export_vars.innodb_buffer_pool_load_incomplete = 0;
+	} else if (!buf_load_abort_flag) {
+		buf_load_status(STATUS_INFO,
+			"Buffer pool(s) load aborted due to user instigated abort at %s",
+			now);
+		/* intentionally don't reset innodb_buffer_pool_load_incomplete
+                   as we don't want a shutdown to save the buffer pool */
+	} else {
+		buf_load_status(STATUS_INFO,
+			"Buffer pool(s) load aborted due to shutdown at %s",
+			now);
+		/* intentionally don't reset innodb_buffer_pool_load_incomplete
+                   as we want to abort without saving the buffer pool */
+	}
 
 	/* Make sure that estimated = completed when we end. */
 	/* mysql_stage_set_work_completed(pfs_stage_progress, dump_n); */
@@ -793,7 +819,7 @@ DECLARE_THREAD(buf_dump_thread)(void*)
 	if (srv_buffer_pool_load_at_startup) {
 
 #ifdef WITH_WSREP
-		if (!wsrep_recovery) {
+		if (!get_wsrep_recovery()) {
 #endif /* WITH_WSREP */
 			buf_load();
 #ifdef WITH_WSREP
@@ -822,15 +848,16 @@ DECLARE_THREAD(buf_dump_thread)(void*)
 	}
 
 	if (srv_buffer_pool_dump_at_shutdown && srv_fast_shutdown != 2) {
+		if (export_vars.innodb_buffer_pool_load_incomplete) {
+			buf_dump_status(STATUS_INFO,
+				"Dumping of buffer pool not started"
+				" as load was incomplete");
 #ifdef WITH_WSREP
-		if (!wsrep_recovery) {
+		} else if (get_wsrep_recovery()) {
 #endif /* WITH_WSREP */
-
-		buf_dump(FALSE /* ignore shutdown down flag,
-		keep going even if we are in a shutdown state */);
-#ifdef WITH_WSREP
+		} else {
+			buf_dump(FALSE/* do complete dump at shutdown */);
 		}
-#endif /* WITH_WSREP */
 	}
 
 	srv_buf_dump_thread_active = false;
